/**
 * Gespenst im Pacman-Spiel.
 * 
 * @author Bernd Gramlich 
 * @version 2012-11-29
 */
public class Gespenst
{
    // Attribute
    private int positionX;
    private int positionY;
    private char blickrichtung;
    private boolean verwundbar;
    private boolean sichtbar;
    private String gesundeFarbe;
    private String krankeFarbe;
    
    // Figuren als Referenzattribute
    private Kreissektor kopf;
    private Rechteck koerper;
    private Kreissektor auge1;
    private Kreissektor auge2;
    private Kreissektor mund;

    /**
     * Konstruktor.
     * Erzeuge ein neues Gespenst mit gesunder Farbe gFarbe und
     * kranker Farbe kFarbe auf dem Levelfeld (x|y).
     */
    public Gespenst (String gFarbe, String kFarbe, int x, int y)
    {
        gesundeFarbe = gFarbe;
        krankeFarbe  = kFarbe;
        
        verwundbar = false;
        sichtbar   = true;
        
        blickrichtung = 'N';
        
        koerper = new Rechteck ();
        koerper.setzeBreite (70);
        koerper.setzeHoehe (65);
        koerper.setzeFarbe (gesundeFarbe);
        koerper.macheSichtbar ();
        
        kopf = new Kreissektor ();
        kopf.setzeRadius (35);
        kopf.setzeAnfangswinkel (0);
        kopf.setzeGesamtwinkel (180);
        kopf.setzeFarbe (gesundeFarbe);
        
        auge1 = new Kreissektor ();
        auge1.setzeRadius (7);
        auge1.setzeAnfangswinkel (0);
        auge1.setzeGesamtwinkel (360);
        auge1.setzeFarbe ("schwarz");
        
        auge2 = new Kreissektor ();
        auge2.setzeRadius (7);
        auge2.setzeAnfangswinkel (0);
        auge2.setzeGesamtwinkel (360);
        auge2.setzeFarbe ("schwarz");
        
        mund = new Kreissektor ();
        mund.setzeRadius (20);
        mund.setzeAnfangswinkel (180);
        mund.setzeGesamtwinkel (180);
        mund.setzeFarbe ("gelb");
               
        bewegeNach (x, y);
    }
    
    /**
     * Drehe das Gespenst nach Westen.
     */
    public void blickeNachWesten ()
    {
       blickrichtung = 'W';
    }
    
    /**
     * Drehe das Gespenst nach Sden.
     */
    public void blickeNachSueden ()
    {
       blickrichtung = 'S';
    }
    
    /**
     * Drehe das Gespenst nach Osten.
     */
    public void blickeNachOsten ()
    {
       blickrichtung = 'O';
    }
    
    /**
     * Drehe das Gespenst nach Norden.
     */
    public void blickeNachNorden ()
    {
       blickrichtung = 'N';
    }


    /**
     * Bewege das Gespenst zu einem bestimmten Levelfeld.
     */
    public void bewegeNach (int x, int y)
    {
        // Wir mssen die Levelkoordinaten in Leinwandkoordinaten umrechnen.
        koerper.setzePosition ((x-1) * 100 + 15, (y-1) * 100 + 35);
        kopf.setzePosition    ((x-1) * 100 + 15, (y-1) * 100     );
        auge1.setzePosition   ((x-1) * 100 + 30, (y-1) * 100 + 30);
        auge2.setzePosition   ((x-1) * 100 + 56, (y-1) * 100 + 30);
        mund.setzePosition    ((x-1) * 100 + 30, (y-1) * 100 + 35);
        
        positionX = x;
        positionY = y;
    }
    
    /**
     * Mache das Gespenst sichtbar.
     */
    public void macheSichtbar ()
    {
        koerper.macheSichtbar ();
        kopf.macheSichtbar ();
        auge1.macheSichtbar ();
        auge2.macheSichtbar ();
        mund.macheSichtbar ();
        
        sichtbar = true;
    }
    
    /**
     * Mache das Gespenst unsichtbar.
     */
    public void macheUnsichtbar ()
    {
        koerper.macheUnsichtbar ();
        kopf.macheUnsichtbar ();
        auge1.macheUnsichtbar ();
        auge2.macheUnsichtbar ();
        mund.macheUnsichtbar ();
        
        sichtbar = false;
    }
    
    /**
     * Mache das Gespenst verwundbar.
     */
    public void werdeVerwundbar ()
    {
        koerper.setzeFarbe (krankeFarbe);
        kopf.setzeFarbe (krankeFarbe);
        
        verwundbar = true;
    }
    
    /**
     * Mache das Gespenst unverwundbar.
     */
    public void werdeUnverwundbar ()
    {
        koerper.setzeFarbe (gesundeFarbe);
        kopf.setzeFarbe (gesundeFarbe);
        
        verwundbar = false;
    }
    
    /**
     * Das Gespenst geht einen Schritt nach vorne. Wenn es dadurch
     * den Spielfeldrand berschreitet, kehrt es auf der anderen
     * Seite des Spielfelds wieder zurck.
     */
    public void schrittInnerhalb ()
    {
        switch (blickrichtung)
        {
            case 'N':
            if (positionY > 1)
            {
                bewegeNach (positionX, positionY - 1);
            }
            else
            {
                bewegeNach (positionX, 7);
            }
            break;
            
            case 'W':
            if (positionX > 1)
            {
                bewegeNach (positionX - 1, positionY);
            }
            else
            {
                bewegeNach (9, positionY);
            }
            break;
            
            case 'S':
            if (positionY < 7)
            {
                bewegeNach (positionX, positionY + 1);
            }
            else
            {
                bewegeNach (positionX, 1);
            }
            break;
            
            case 'O': 
            if (positionX < 9)
            {
                bewegeNach (positionX + 1, positionY);
            }
            else
            {
                bewegeNach (1, positionY);
            }
            break;
        }
    }
    
    /**
     * Jage den Pacman.
     * Das Gespenst berechnet die krzeste Entfernung zum Zielpunkt
     * mit den Levelkoordinaten (zielX|zielY). Anschlieend dreht
     * und bewegt sich das Gespenst so, dass die Entfernung noch
     * krzer wird.
     */
    public void jage (int zielX, int zielY)
    {
        /* 
         * Rechne aus, wie viele Felder das Ziel in den einzelnen Himmelsrichtungen
         * entfernt ist.
         */
        int nachOstenZumZiel  = zielX - positionX;
        int nachWestenZumZiel = positionX - zielX;
        int nachSuedenZumZiel = zielY - positionY;
        int nachNordenZumZiel = positionY - zielY;
        
        if (nachOstenZumZiel < 0)
        {
            nachOstenZumZiel += 9;
        }
        
        if (nachWestenZumZiel < 0)
        {
            nachWestenZumZiel += 9;
        }
        
        if (nachSuedenZumZiel < 0)
        {
            nachSuedenZumZiel += 7;
        }
        
        if (nachNordenZumZiel < 0)
        {
            nachNordenZumZiel += 7;
        }
        
        /*
         * Die Ost-West-Entfernung ist die kleinere der beiden Zahlen
         * nachOstenZumZiel und nachWestenZumZiel.
         * Die Nord-Sd-Entfernung ist die kleinere der beiden Zahlen
         * nachNordenZumZiel und nachSuedenZumZiel.
         */
        int ostwest;
        int nordsued;
        
        if (nachWestenZumZiel < nachOstenZumZiel)
        {
            ostwest = nachWestenZumZiel;
        }
        else
        {
            ostwest = nachOstenZumZiel;
        }
        
        if (nachSuedenZumZiel < nachNordenZumZiel)
        {
            nordsued = nachSuedenZumZiel;
        }
        else
        {
            nordsued = nachNordenZumZiel;
        }
        
        /*
         * Wenn die Ost-West-Entfernung grer ist, jagen wir das Ziel in
         * Ost-West-Richtung. Wenn die Nord-Sd-Entfernung grer ist,
         * jagen wir das Ziel in Nord-Sd-Richtung. Wenn beide Entfernung
         * gleich sind, entscheiden wir uns zufllig zwischen
         * Ost-West-Richtung und Nord-Sd-Richtung.
         */
        if (ostwest > nordsued)
        {
            if (nachWestenZumZiel < nachOstenZumZiel)
            {
                blickeNachWesten ();
            }
            else
            {
                blickeNachOsten ();
            }
        }
        else if (ostwest < nordsued)
        {
            if (nachNordenZumZiel < nachSuedenZumZiel)
            {
                blickeNachNorden ();
            }
            else
            {
                blickeNachSueden ();
            }
        }
        else // ostwest == nordsued
        {
            if (Math.random () < 0.5)
            {
                if (nachWestenZumZiel > nachOstenZumZiel)
                {
                    blickeNachWesten ();
                }
                else
                {
                    blickeNachOsten ();
                }
            }
            else
            {
                if (nachNordenZumZiel > nachSuedenZumZiel)
                {
                    blickeNachNorden ();
                }
                else
                {
                    blickeNachSueden ();
                }
            }
        }
        
        /*
         * Wenn wir nicht bereits auf dem Ziel stehen (also alle Entfernungen 0
         * sind), dann gehen wir einen Schritt in die gewhlte Richtung.
         */
        if (ostwest + nordsued > 0)
        {
            schrittInnerhalb ();
        }
    } 
    
    /**
     * Nenne die x-Koordinate des Gespenstes im Level.
     */
    public int gibX ()
    {
        return positionX;
    }
    
    /**
     * Nenne die y-Koordinate des Gespenstes im Level.
     */
    public int gibY ()
    {
        return positionY;
    }
    
    /**
     * Gib an, ob das Gespenst verwundbar ist.
     */
    public boolean istVerwundbar ()
    {
        return verwundbar;
    }
}
