/**
 * Ein einfaches Pacman-Spiel.
 * Der Pacman muss in einer leeren Welt vor den vier Gespenstern flchten.
 * 
 * @author Bernd Gramlich 
 * @version 2012-11-29
 */

import java.awt.event.KeyEvent;

public class Spiel
{
	private Vielfrass pacman;
	private Gespenst blinky, pinky, inky, clyde;
	private Beschriftung ueberschrift;
	private Waechter spielschutz;
	
	/**
	 * Konstruktor.
	 * Erzeuge die vier Gespenster und den Pacman und bringe sie in Ausgangsposition.
	 */
	public Spiel ()
	{
		blinky = new Gespenst ("rot", "blau", 1, 1);
		pinky  = new Gespenst ("pink", "blau", 1, 1);
		inky   = new Gespenst ("trkis", "blau", 1, 1);
		clyde  = new Gespenst ("orange", "blau", 1, 1);
		pacman = new Vielfrass ();
		
		ueberschrift = new Beschriftung ();
		spielschutz  = new Waechter ();
		
		erzeugeAusgangsposition ();
	}

	/**
	 * Erzeuge die Ausgangsposition des Pacmanspiels.
	 * Der Pacman sitzt in der Mitte der Welt, die vier Gespenster an den Ecken.
	 */
	public void erzeugeAusgangsposition ()
	{
		blinky.bewegeNach (1, 1);
		pinky.bewegeNach  (9, 1);
		inky.bewegeNach   (9, 7);
		clyde.bewegeNach  (1, 7);
		pacman.bewegeNach (5, 4);
		
		blinky.macheSichtbar ();
		pinky.macheSichtbar ();
		inky.macheSichtbar ();
		clyde.macheSichtbar ();
		pacman.macheSichtbar ();
		
		ueberschrift.setzeInhalt ("  Pacman  ");
		ueberschrift.setzeFarbe ("grau");
		ueberschrift.setzeSchriftgroesse (80);
		ueberschrift.setzePosition (300, 300);
		ueberschrift.macheSichtbar ();		
	}
	
	/**
	 * Spiele.
	 * Der Pacman horcht auf Tastaturbefehle mit den Pfeiltasten.
	 * Die Gespenster verfolgen ihn mit ihrer Methode jage ().
	 * Spielende ist, wenn die Gespenster den Pacman gefangen
	 * haben oder wenn die Eingabetaste gedrckt wird.
	 */
	public void spiele ()
	{
	    /*
	     * Hilfsvariablen:
	     * spielfeld:  Die Leinwand, auf der wir spielen.
	     * aufhoeren:  Soll das Spiel jetzt beendet werden?
	     * bewegen:    Sollen die Figuren jetzt bewegt werden?
	     * taste:      Die letzte Taste, die auf der Leinwand gedrckt wurde (als
	     *             Tastaturereignis)
	     */
	    Leinwand spielfeld = Leinwand.gibLeinwand ();
        boolean aufhoeren = false;
        boolean bewegen;
        KeyEvent taste;
        
        /*
         * Lsche alle Tasten, die vor Spielbeginn gedrckt wurden.
         */
        while ((taste = spielfeld.tastendruck ()) != null);
        
        /*
         * Bringe die Figuren in Ausgangsposition, warte eine Sekunde
         * und mache dann die berschrift unsichtbar.
         */
        erzeugeAusgangsposition ();
        spielfeld.warte (1000);
        ueberschrift.macheUnsichtbar ();
        
        /*
         * Erfrage Passwort.
         */
        ueberschrift.setzeInhalt (" Passwort?");
		ueberschrift.macheSichtbar ();
		spielschutz.warteAufErlaubnis ();
		ueberschrift.macheUnsichtbar ();
        
        /*
         * Wiederhole, so lange wir nicht aufhren mssen.
         */
        while (!aufhoeren)
        {
            /*
             * Solange keine Taste gedrckt wird, tue nichts.
             */
            while ((taste = spielfeld.tastendruck ()) == null);
            
            /*
             * Reagiere unterschiedlich, je nachdem, welche Taste gedrckt wurde.
             */
            switch (taste.getKeyCode())
            {
                /*
                 * Wenn eine der Tasten Pfeil nach oben, Pfeil nach links, Pfeil nach
                 * rechts oder Pfeil nach unten gedrckt wurde, dann wollen wir die
                 * Figuren bewegen. Wir drehen den Pacman in die passende
                 * Himmelsrichtung.
                 */
                case KeyEvent.VK_UP:    bewegen = true; pacman.blickeNachNorden (); break;
                case KeyEvent.VK_LEFT:  bewegen = true; pacman.blickeNachWesten (); break;
                case KeyEvent.VK_DOWN:  bewegen = true; pacman.blickeNachSueden (); break;
                case KeyEvent.VK_RIGHT: bewegen = true; pacman.blickeNachOsten  (); break;
                /*
                 * Wenn die Eingabetaste gedrckt wurde, wollen wir die Figuren nicht mehr
                 * bewegen, sondern mit dem Spielen aufhren.
                 */
                case KeyEvent.VK_ENTER: bewegen = false; aufhoeren = true; break;
                /*
                 * Wenn irgendeine andere Taste gedrckt wurde, bewegen wir die Figuren
                 * nicht.
                 */
                default:                bewegen = false;
            }
            
            /*
             * Wenn die Figuren bewegt werden sollen ...
             */
            if (bewegen)
            {
                /*
                 * ... dann macht zuerst der Pacman einen Schritt in seine Blickrichtung.
                 * Anschlieend versuchen alle vier Gespenster, ihn zu fangen.
                 */
                pacman.schrittInnerhalb ();
                blinky.jage (pacman.gibX (), pacman.gibY());
                pinky.jage  (pacman.gibX (), pacman.gibY());
                inky.jage   (pacman.gibX (), pacman.gibY());
                clyde.jage  (pacman.gibX (), pacman.gibY());
                
                if (   (blinky.gibX () == pacman.gibX ()
                        && blinky.gibY () == pacman.gibY ())
                    || (pinky.gibX () == pacman.gibX ()
                        && pinky.gibY () == pacman.gibY ())
                    || (inky.gibX () == pacman.gibX ()
                        && inky.gibY () == pacman.gibY ())                  
                    || (clyde.gibX () == pacman.gibX ()
                        && clyde.gibY () == pacman.gibY ()))
                {
                    aufhoeren = true;
                }
            }
        }
        
        /* Zu Spielende geben wir eine Nachricht aus, warten 3 Sekunden und machen dann
         * alle Figuren und die Nachricht unsichtbar.
         */
        ueberschrift.setzeInhalt ("Game over!");
		ueberschrift.macheSichtbar ();
		spielfeld.warte (3000);
		blinky.macheUnsichtbar ();
		pinky.macheUnsichtbar ();
		inky.macheUnsichtbar ();
		clyde.macheUnsichtbar ();
		pacman.macheUnsichtbar ();
		ueberschrift.macheUnsichtbar ();
	}
}