/**
 * Der Vielfrass ist die Hauptfigur im Pacman-Spiel.
 * 
 * @author Bernd Gramlich 
 * @version 2012-11-29
 */
public class Vielfrass
{
    // Attribute
    private int positionX;
    private int positionY;
    private boolean verwundbar;
    private boolean sichtbar;
    private char blickrichtung;
    
    // Referenzattribut
    // Das Bild unseres Vielfrass ist ein Kreissektor.
    private Kreissektor bild;

    /**
     * Konstruktor.
     * Erzeugt einen Vielfrass in Standardposition und -richtung.
     */
    public Vielfrass()
    {
        positionX     = 1;
        positionY     = 1;
        verwundbar    = true;
        sichtbar      = true;
        blickrichtung = 'O';
        
        // Erzeuge das Bild des Vielfrass
        bild = new Kreissektor ();
    }

    /**
     * Drehe den Vielfrass nach Westen.
     */
    public void blickeNachWesten ()
    {
       bild.setzeAnfangswinkel (210);
       
       blickrichtung = 'W';
    }
    
    /**
     * Drehe den Vielfrass nach Sden.
     */
    public void blickeNachSueden ()
    {
       bild.setzeAnfangswinkel (300);
       
       blickrichtung = 'S';
    }
    
    /**
     * Drehe den Vielfrass nach Osten.
     */
    public void blickeNachOsten ()
    {
       bild.setzeAnfangswinkel (30);
       
       blickrichtung = 'O';
    }
    
    /**
     * Drehe den Vielfrass nach Norden.
     */
    public void blickeNachNorden ()
    {
       bild.setzeAnfangswinkel (120);
       
       blickrichtung = 'N';
    }
    
    /**
     * Mache den Vielfrass unverwundbar.
     */
    public void werdeUnverwundbar ()
    {
        verwundbar = false;
        
        bild.setzeFarbe ("rot");
    }
    
    /**
     * Mache den Vielfrass verwundbar.
     */
    public void werdeVerwundbar ()
    {
        verwundbar = true;
        
        bild.setzeFarbe ("gelb");
    }
    
    /**
     * Mache den Vielfrass sichtbar.
     */
    public void macheSichtbar ()
    {
        bild.macheSichtbar ();
        
        sichtbar = true;
    }
    
    /**
     * Mache den Vielfrass unsichtbar.
     */
    public void macheUnsichtbar ()
    {
        bild.macheUnsichtbar ();
        
        sichtbar = false;
    }
    

    
    /**
     * Drehe den Vielfra nach links.
     */
    public void dreheNachLinks ()
    {
        switch (blickrichtung)
        {
            case 'N': blickeNachWesten (); break;
            case 'W': blickeNachSueden (); break;
            case 'S': blickeNachOsten (); break;
            case 'O': blickeNachNorden (); break;
        }
    }
    
    /**
     * Drehe den Vielfra nach rechts.
     */
    public void dreheNachRechts ()
    {
        switch (blickrichtung)
        {
            case 'N': blickeNachOsten (); break;
            case 'W': blickeNachNorden (); break;
            case 'S': blickeNachWesten (); break;
            case 'O': blickeNachSueden (); break;
        }
    }   

    /**
     * Nenne die Blickrichtung des Vielfra.
     */
    public void schreibeBlickrichtung ()
    {
        switch (blickrichtung)
        {
            case 'N': System.out.printf ("Hallo, Welt! Ich schaue nach Norden!\n"); break;
            case 'W': System.out.printf ("Hallo, Welt! Ich schaue nach Westen!\n"); break;
            case 'S': System.out.printf ("Hallo, Welt! Ich schaue nach Sueden!\n"); break;
            case 'O': System.out.printf ("Hallo, Welt! Ich schaue nach Osten!\n"); break;
        }
    }
    
    /**
     * Bewege den Vielfra zu einem bestimmten Levelfeld.
     */
    public void bewegeNach (int x, int y)
    {
        // Wir mssen die Levelkoordinaten in Leinwandkoordinaten umrechnen.
        bild.setzePosition (x * 100 - 99, y * 100 - 99);
        
        positionX = x;
        positionY = y;
    }
    
    /**
     * Der Vielfra geht einen Schritt nach vorne.
     */
    public void schritt ()
    {
        switch (blickrichtung)
        {
            case 'N': bewegeNach (positionX, positionY - 1); break;
            case 'W': bewegeNach (positionX - 1, positionY); break;
            case 'S': bewegeNach (positionX, positionY + 1); break;
            case 'O': bewegeNach (positionX + 1, positionY); break;
        }
    }
    
    /**
     * Der Vielfra geht einen Schritt zurck.
     */
    public void schrittRueckwaerts ()
    {
        switch (blickrichtung)
        {
            case 'N': bewegeNach (positionX, positionY + 1); break;
            case 'W': bewegeNach (positionX + 1, positionY); break;
            case 'S': bewegeNach (positionX, positionY - 1); break;
            case 'O': bewegeNach (positionX - 1, positionY); break;
        }
    }
    
    /**
     * Der Vielfra geht einen Schritt nach vorne. Wenn er dadurch
     * den Spielfeldrand berschreitet, kehrt er auf der anderen
     * Seite des Spielfelds wieder zurck.
     */
    public void schrittInnerhalb ()
    {
        switch (blickrichtung)
        {
            case 'N':
            if (positionY > 1)
            {
                bewegeNach (positionX, positionY - 1);
            }
            else
            {
                bewegeNach (positionX, 7);
            }
            break;
            
            case 'W':
            if (positionX > 1)
            {
                bewegeNach (positionX - 1, positionY);
            }
            else
            {
                bewegeNach (9, positionY);
            }
            break;
            
            case 'S':
            if (positionY < 7)
            {
                bewegeNach (positionX, positionY + 1);
            }
            else
            {
                bewegeNach (positionX, 1);
            }
            break;
            
            case 'O': 
            if (positionX < 9)
            {
                bewegeNach (positionX + 1, positionY);
            }
            else
            {
                bewegeNach (1, positionY);
            }
            break;
        }
    }
        
    /**
     * Nenne die x-Koordinate des Vielfraes im Level.
     */
    public int gibX ()
    {
        return positionX;
    }
    
    /**
     * Nenne die y-Koordinate des Vielfraes im Level.
     */
    public int gibY ()
    {
        return positionY;
    }
    
    /**
     * Gib an, ob der Vielfra verwundbar ist.
     */
    public boolean istVerwundbar ()
    {
        return verwundbar;
    }
}
        